#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <glib.h>

#include <pan/base/article.h>
#include <pan/base/base-prefs.h>

#include <pan/filters/filter.h>
#include <pan/filters/filter-aggregate.h>
#include <pan/filters/filter-binary.h>
#include <pan/filters/filter-cached.h>
#include <pan/filters/filter-crosspost.h>
#include <pan/filters/filter-mine.h>
#include <pan/filters/filter-date.h>
#include <pan/filters/filter-linecount.h>
#include <pan/filters/filter-new.h>
#include <pan/filters/filter-phrase.h>
#include <pan/filters/filter-score.h>

static void
init_pan (void)
{
	base_prefs_init (g_get_tmp_dir(), ">", 100, FALSE, FALSE);
}

extern int server_saving_enabled;

static int
test_phrases (void)
{
	Article * article;
	GPtrArray * articles;
	Server * server;
	Group * group;
	static gulong number = 0;
	int test = 0;
	Filter * f = filter_phrase_new ();

	server = server_new ();
	server->name = "dummy_server";
	group = group_new (server, "dummy_group");
	group_set_is_folder (group, TRUE);
	articles = g_ptr_array_new ();

	article = article_new (group);
	article->number = ++number;
	article_set_message_id  (article, "<asdf@foobar.com>");

#define test(subject,expected_result) \
	article_set_subject (article, subject); \
	if (!!filter_test_article (f, article) != !!expected_result) return 1; \
	g_message ("PASS: filter-phrase %d", ++test);

	filter_phrase_set (FILTER_PHRASE(f), PHRASE_MATCH_CONTAINS, PHRASE_KEY_SUBJECT, "fillyjonk", FALSE);
	test ("Can we find fillyjonk when it's by itself?", TRUE)
	test ("Can we findfillyjonkwhen it's inside a string?", TRUE)
	test ("WTF is a fillyjonk?", TRUE)
	test ("WTF is a fillyjon?", FALSE)

	filter_phrase_set (FILTER_PHRASE(f), PHRASE_MATCH_STARTS_WITH, PHRASE_KEY_SUBJECT, "fillyjonk", FALSE);
	test ("fillyjonk at the front", TRUE)
	test ("Fillyjonk at the front, in Caps", TRUE)
	test ("at the end comes the fillyjonk", FALSE)
	test ("the fillyjonk comes before the mymble", FALSE)

	filter_phrase_set (FILTER_PHRASE(f), PHRASE_MATCH_REGEX, PHRASE_KEY_SUBJECT, "^fillyjonk", FALSE);
	test ("fillyjonk at the front", TRUE)
	test ("Fillyjonk at the front, in Caps", TRUE)
	test ("at the end comes the fillyjonk", FALSE)
	test ("the fillyjonk comes before the mymble", FALSE)

	filter_phrase_set (FILTER_PHRASE(f), PHRASE_MATCH_ENDS_WITH, PHRASE_KEY_SUBJECT, "fillyjonk", FALSE);
	test ("fillyjonk at the front", FALSE)
	test ("at the end comes the fillyjonk", TRUE)
	test ("at the end, in caps, comes the Fillyjonk", TRUE)
	test ("the fillyjonk comes before the mymble", FALSE)

	filter_phrase_set (FILTER_PHRASE(f), PHRASE_MATCH_REGEX, PHRASE_KEY_SUBJECT, "fillyjonk$", FALSE);
	test ("fillyjonk at the front", FALSE)
	test ("at the end comes the fillyjonk", TRUE)
	test ("at the end, in caps, comes the Fillyjonk", TRUE)
	test ("the fillyjonk comes before the mymble", FALSE)

	filter_phrase_set (FILTER_PHRASE(f), PHRASE_MATCH_IS, PHRASE_KEY_SUBJECT, "fillyjonk", FALSE);
	test ("fillyjonk at the front", FALSE)
	test ("at the end comes the fillyjonk", FALSE)
	test ("at the end, in caps, comes the Fillyjonk", FALSE)
	test ("the fillyjonk comes before the mymble", FALSE)
	test (" fillyjonk ", FALSE)
	test ("fillyjonk ", FALSE)
	test ("fillyjonk", TRUE)
	test ("illyjonk", FALSE)
	test ("fillyjonk ", FALSE)

	filter_phrase_set (FILTER_PHRASE(f), PHRASE_MATCH_REGEX, PHRASE_KEY_SUBJECT, "^fillyjonk$", FALSE);
	test ("fillyjonk at the front", FALSE)
	test ("at the end comes the fillyjonk", FALSE)
	test ("at the end, in caps, comes the Fillyjonk", FALSE)
	test ("the fillyjonk comes before the mymble", FALSE)
	test (" fillyjonk ", FALSE)
	test ("fillyjonk ", FALSE)
	test ("fillyjonk", TRUE)
	test ("illyjonk", FALSE)
	test ("fillyjonk ", FALSE)

	pan_object_unref (PAN_OBJECT (f));

	return 0;
}

int
main (void)
{
	gint val;
	g_thread_init (NULL);
	init_pan ();
	server_saving_enabled = FALSE;

	val = test_phrases ();
	if (val)
		return val;

	g_message ("All filter tests passed");
	return 0;
}
